import sqlite from "sqlite3";
import bcrypt from "bcryptjs";
import { PHASE_PRODUCTION_BUILD } from "next/dist/shared/lib/constants";

const { CREDENTIALS_DB_PATH } = process.env;

export const database = () => {
  if (!CREDENTIALS_DB_PATH && process.env.NEXT_PHASE != PHASE_PRODUCTION_BUILD) {
    throw "$CREDENTIALS_DB_PATH not provided; unable to connect to database";
  }

  return new sqlite.Database(CREDENTIALS_DB_PATH!, (err: any) => {
    if (err && process.env.NEXT_PHASE != PHASE_PRODUCTION_BUILD) throw err;
  });
};

export function validateCredentials(email: string, password: string) {
  const db = database();

  return new Promise<boolean>((resolve, reject) => {
    db.get(
      "SELECT key, value FROM passwords WHERE key = ?",
      email,
      async (err, row: any) => {
        db.close(); // We don't need this anymore

        if (err) return reject(err);
        if (!row) return resolve(false);

        try {
          const hash: string = row.value.replace("bcrypt:", "");
          const isValid = await bcrypt.compare(password, hash);
          resolve(isValid);
        } catch(e) {
          reject(e);
        }
      }
    );
  });
}

export function setUserPassword(email: string, newPass: string) {
  return new Promise<void>(async (resolve, reject) => {
    const db = database();
    const hash = 'bcrypt:' + await bcrypt.hash(newPass, 10);

    db.run("UPDATE passwords SET value = $1 WHERE key = $2",
    { 1: hash, 2: email },
    async (err: any, res: any) => {
      db.close();
      if (err) return reject(err);
      console.log(res);
      resolve();
    });
  });
}
